import slumber
import re
from django.db import models
from candideitorg.apikey_auth import ApiKeyAuth
from slumber import url_join, Resource
from django.conf import settings
import django.dispatch
from django.utils.translation import ugettext as _


twitter_regexp = re.compile(r"^https?://[^/]*(t\.co|twitter\.com)(/.*|/?)")
facebook_regexp = re.compile(r"^https?://[^/]*(facebook\.com|fb\.com|fb\.me)(/.*|/?)")


election_finished = django.dispatch.Signal(providing_args=[])

# Create your models here.
class CandideitorgDocument(models.Model):
    remote_id = models.IntegerField()
    resource_uri = models.CharField(max_length=255)

    @classmethod
    def get_api(cls):
        api = slumber.API(settings.CANDIDEITORG_URL, auth=ApiKeyAuth(settings.CANDIDEITORG_USERNAME, settings.CANDIDEITORG_API_KEY))
        return api

    class Meta:
	   abstract = True

    @classmethod
    def get_resource_as_dict(cls, uri):
        kwargs = {}
        api = CandideitorgDocument.get_api()
        for key, value in api._store.iteritems():
            kwargs[key] = value
        kwargs.update({"base_url": url_join(api._store["base_url"], uri)})
        resource = Resource(**kwargs)
        dicti = resource.get()
        return dicti

    @classmethod
    def create_new_from_dict(cls, dicti, **kwargs):
        field_names = []

        for field in  iter(cls._meta.fields):
            field_names.append(field.name)

        new_element = {}
        for key in dicti.keys():
            if not isinstance(dicti[key], (list, tuple)):
                if key in field_names and key != "id":
                    new_element[key]= dicti[key]
        new_element["remote_id"] = dicti["id"]

        new_element.update(kwargs)
        return cls.get_or_create(dicti, new_element)

    @classmethod
    def get_or_create(cls, dictified_element, new_element):
        existing_objects = cls.objects.filter(remote_id=dictified_element["id"]).count()
        if existing_objects > 0:
            return cls.objects.get(remote_id=dictified_element["id"])
        else:
            return cls.objects.create(**new_element)




class Election(CandideitorgDocument):
    information_source = models.TextField(null=True, blank=True)
    description = models.TextField(null=True, blank=True)
    name = models.CharField(max_length=255, blank=True)
    logo = models.CharField(max_length=255, null=True, blank=True)
    use_default_media_naranja_option = models.BooleanField(default=False)
    slug = models.SlugField(max_length=255)

    def __unicode__(self):
        return self.name

    @classmethod
    def get_or_create(cls, dictified_element, new_element):
        existing_objects = cls.objects.filter(remote_id=dictified_element["id"]).count()
        created = True
        if existing_objects > 0:
            election = cls.objects.get(remote_id=dictified_element["id"])
            created = False
        else:
            election = cls.objects.create(**new_element)
        
        return election, created

    def update_answers(self):
        for candidate in self.candidate_set.all():
            candidate_dictionary = CandideitorgDocument.get_resource_as_dict(candidate.resource_uri)
            PersonalDataCandidate.objects.filter(candidate=candidate).delete()
            for uri in candidate_dictionary['personal_data_candidate']:

                pdc_dictionary = CandideitorgDocument.get_resource_as_dict(uri)
                try:
                    personaldata = PersonalData.objects.get(resource_uri=pdc_dictionary["personal_data"])
                    PersonalDataCandidate.create_new_from_dict(pdc_dictionary, 
                                                            candidate=candidate, 
                                                            personaldata=personaldata)
                except:
                    pass




        questions = Question.objects.filter(category__election=self)

        for question in questions:
            question_dictionary = CandideitorgDocument.get_resource_as_dict(question.resource_uri)
            question = Question.objects.get(resource_uri=question_dictionary['resource_uri'])
            for answer_uri in question_dictionary['answers']:
                answer_dictionary = CandideitorgDocument.get_resource_as_dict(answer_uri)

                answer = Answer.create_new_from_dict(answer_dictionary,question=question)


                if answer.candidate_set.all():
                    answer.candidate_set.clear()

                new_candidates = []

                for candidate_uri in answer_dictionary['candidates']:
                    candidate = Candidate.objects.get(resource_uri=candidate_uri)
                    new_candidates.append(candidate)


                answer.candidate_set = new_candidates
            #Getting the information sources
            for information_source_uri in question_dictionary['information_sources']:
                information_dictionary = CandideitorgDocument.get_resource_as_dict(information_source_uri)
                try:
                    information_source = InformationSource.objects.get(resource_uri=information_source_uri)
                    information_source.content = information_dictionary['content']
                    information_source.save()
                except InformationSource.DoesNotExist, error:
                    #I have to create it with something like this 
                    # information_source = InformationSource.create_new_from_dict(dictionary, question=question, candidate=candidate)
                    candidate = Candidate.objects.get(resource_uri=information_dictionary['candidate'])
                    information_source = InformationSource.create_new_from_dict(information_dictionary, question=question, candidate=candidate)






    def cleanup_before_updating(self):
        Candidate.objects.filter(election=self).delete()
        Question.objects.filter(category__election=self).delete()

    def update(self):
        self.cleanup_before_updating()
        api = self.__class__.get_api()
        election_dict = api.election(self.remote_id).get()
        election, created = Election.create_new_from_dict(election_dict)
        for uri in election_dict['personal_data']:
            dictionary = CandideitorgDocument.get_resource_as_dict(uri)
            PersonalData.create_new_from_dict(dictionary, election=election)
        for uri in election_dict['background_categories']:
            dictionary = CandideitorgDocument.get_resource_as_dict(uri)
            backgroundcategory = BackgroundCategory.create_new_from_dict(dictionary, election=election)
            for uri in dictionary['background']:
                dictionary = CandideitorgDocument.get_resource_as_dict(uri)
                Background.create_new_from_dict(dictionary, background_category=backgroundcategory)
        for uri in election_dict['candidates']:
            candidate_dictionary = CandideitorgDocument.get_resource_as_dict(uri)
            candidate = Candidate.create_new_from_dict(candidate_dictionary, election=election)
            for uri in candidate_dictionary['personal_data_candidate']:
                pdc_dictionary = CandideitorgDocument.get_resource_as_dict(uri)
                personal_data = PersonalData.objects.get(resource_uri=pdc_dictionary["personal_data"])
                #delete previous
                PersonalDataCandidate.objects.filter(candidate=candidate, personaldata=personal_data).delete()
                #delete previous
                PersonalDataCandidate.create_new_from_dict(pdc_dictionary, 
                    candidate=candidate, 
                    personaldata=personal_data)
            for uri in candidate_dictionary['links']:
                dictionary = CandideitorgDocument.get_resource_as_dict(uri)
                Link.create_new_from_dict(dictionary, candidate=candidate)
            for uri in candidate_dictionary['backgrounds_candidate']:
                dictionary = CandideitorgDocument.get_resource_as_dict(uri)
                background = Background.objects.get(resource_uri=dictionary["background"])
                BackgroundCandidate.create_new_from_dict(dictionary, 
                    candidate=candidate, 
                    background=background)
        for uri in election_dict['categories']:
            dictionary = CandideitorgDocument.get_resource_as_dict(uri)
            category = Category.create_new_from_dict(dictionary, election=election)
            for uri in dictionary['questions']:

                question_dictionary = CandideitorgDocument.get_resource_as_dict(uri)
                question = Question.create_new_from_dict(question_dictionary, category=category)
                for uri in question_dictionary['answers']:
                    dictionary = CandideitorgDocument.get_resource_as_dict(uri)
                    answer = Answer.create_new_from_dict(dictionary,question=question)
                    answer.candidate_set.clear()
                    for candidate_uri in dictionary["candidates"]:
                        candidate = Candidate.objects.get(resource_uri=candidate_uri)
                        question_of_the_answer = answer.question
                        candidate.answers.add(answer)
                        candidate.save()

                for uri in question_dictionary['information_sources']:
                    dictionary = CandideitorgDocument.get_resource_as_dict(uri)
                    candidate = Candidate.objects.get(resource_uri=dictionary['candidate'])
                    information_source = InformationSource.create_new_from_dict(dictionary, question=question, candidate=candidate)

    @classmethod
    def fetch_all_from_api(cls,offset=0, max_elections=None):
        api = cls.get_api()
        elections_from_api = api.election.get(offset=offset)
        meta = elections_from_api['meta']
        counter = 0
        while True:
            
            for election_dict in elections_from_api["objects"]:
                election, created = Election.create_new_from_dict(election_dict)
                counter += 1
                if max_elections and max_elections == counter:
                    return    
                election.update()
                election_finished.send(sender=Election, instance=election, created=created)
                    
            offset = meta["offset"] + meta["limit"]
            if not meta["next"]:
                break
            elections_from_api = api.election.get(offset=offset, limit=meta["limit"])
            meta = elections_from_api['meta']


class Category(CandideitorgDocument):
    name = models.CharField(max_length=255)
    order = models.IntegerField()
    election = models.ForeignKey(Election)
    slug = models.SlugField(max_length=255)

    def __unicode__(self):
        return _(u'"%(category)s" in election "%(election)s"') % {
        "category": self.name,
        "election": self.election.name
        }

class Candidate(CandideitorgDocument):
    photo = models.CharField(max_length=255, null=True, blank=True)
    name = models.CharField(max_length=255)
    slug = models.SlugField(max_length=255)
    has_answered = models.BooleanField()
    election = models.ForeignKey(Election)
    answers = models.ManyToManyField('Answer', null=True, blank=True)

    def __unicode__(self):
        return self.name


class BackgroundCategory(CandideitorgDocument):
    election = models.ForeignKey(Election)
    name = models.CharField(max_length=255)

class PersonalData(CandideitorgDocument):
    label = models.CharField(max_length=255)
    election = models.ForeignKey(Election)

class Background(CandideitorgDocument):
    name = models.CharField(max_length=255)
    background_category = models.ForeignKey(BackgroundCategory)

class Answer(CandideitorgDocument):
    caption = models.CharField(max_length=255)
    question = models.ForeignKey('Question')
    # candidate = models.ForeignKey(Candidate)

    def __unicode__(self):
        return "'%(answer)s' for '%(question)s'"%{
            "answer":self.caption,
            "question":self.question.question,
            }

class Question(CandideitorgDocument):
    question = models.CharField(max_length=255)
    category = models.ForeignKey(Category)

    def __unicode__(self):
        return _(u'"%(question)s" in category "%(category)s", in election "%(election)s"') % {
        'question': self.question,
        'category': self.category.name,
        'election': self.category.election.name
        }
    def _get_number_of_answers_(self):
        answers = self.answer_set.all()
        number_of_answers = 0
        for answer in answers:
            number_of_answers += answer.candidate_set.all().count()
        return number_of_answers
        
    number_of_answers = property(_get_number_of_answers_)




class InformationSource(CandideitorgDocument):
    question = models.ForeignKey(Question)
    candidate = models.ForeignKey(Candidate)
    content = models.TextField()

    def __unicode__(self):
        return _(u'Reference for "%(candidate)s" in question "%(question)s"') % {
        'question':self.question.question,
        'candidate':self.candidate.name

        }
class PersonalDataCandidate(CandideitorgDocument):
    value = models.CharField(max_length=255, null=True)
    candidate = models.ForeignKey(Candidate)
    personaldata = models.ForeignKey(PersonalData)

class Link(CandideitorgDocument):
    url = models.URLField(max_length=255)
    name = models.CharField(max_length=255)
    candidate = models.ForeignKey(Candidate)

    @property
    def icon_class(self):
        if twitter_regexp.match(self.url):
            return "icon-twitter-sign"
        elif facebook_regexp.match(self.url):
            return "icon-facebook-sign"
        else:
            return "icon-globe"


class BackgroundCandidate(CandideitorgDocument):
    value = models.CharField(max_length=255)
    candidate = models.ForeignKey(Candidate)
    background = models.ForeignKey(Background)



from django.db import models
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType


class Report(models.Model):
    owner = models.ForeignKey('auth.User', null=True)
    reason = models.TextField(max_length=512)
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    @models.permalink
    def get_absolute_url(self):
        return ('report_detail', None, {'pk': self.pk})



from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.forms import formsets
from django.http import HttpResponse, Http404
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.template import RequestContext
from django.template.context import RequestContext
from django.utils import simplejson as json
from django.utils.decorators import method_decorator
from django.views.decorators.http import require_http_methods
from django.views.generic import CreateView, DetailView, UpdateView

# Import forms
from elections.forms import PersonalDataForm, PersonalDataCandidateForm

# Import models
from elections.models import Election, Candidate, PersonalData


# PersonalData Views
class PersonalDataCreateView(CreateView):
    model = PersonalData
    form_class = PersonalDataForm
    
    def get_template_names(self):
        return ['elections/wizard/step_three.html']

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(PersonalDataCreateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(PersonalDataCreateView, self).get_context_data(**kwargs)
        context['election'] = get_object_or_404(Election, slug=self.kwargs['election_slug'], owner=self.request.user)
        return context

    def get_success_url(self):
        return reverse('personal_data_create', kwargs={'election_slug': self.kwargs['election_slug']})

    def form_valid(self, form):
        self.object = form.save(commit=False)
        election = get_object_or_404(Election, slug=self.kwargs['election_slug'], owner=self.request.user)
        self.object.election = election
        return super(PersonalDataCreateView, self).form_valid(form)

@login_required
@require_http_methods(['GET', 'POST'])
def personal_data_candidate_create(request, candidate_pk, personal_data_pk):
    candidate = get_object_or_404(Candidate, pk=candidate_pk, election__owner=request.user)
    personal_data = get_object_or_404(PersonalData, pk=personal_data_pk, election__owner=request.user)

    if request.POST:
        value = request.POST.get('value', None)
        candidate.add_personal_data(personal_data, value)

        return HttpResponse(json.dumps({"value": value}),
                            content_type='application/json')
    form = PersonalDataCandidateForm()
    return render_to_response(\
            'elections/personal_data_associate.html',
            {'candidate': candidate, 'personal_data': personal_data, 'form':form},
            context_instance=RequestContext(request))


@login_required
@require_http_methods(['POST'])
def async_delete_personal_data(request, personal_data_pk):
    personal_data = get_object_or_404(PersonalData, pk = personal_data_pk, election__owner=request.user)
    personal_data.delete()
    json_dictionary = {"result":"OK"}
    return HttpResponse(json.dumps(json_dictionary),content_type='application/json')


@login_required
@require_http_methods(['POST'])
def async_create_personal_data(request, election_pk):
    election = get_object_or_404(Election, pk=election_pk, owner=request.user)

    value = request.POST.get('value', None)
    personal_data = PersonalData(label=value, election=election)
    personal_data.save()
    return HttpResponse(json.dumps({"pk": personal_data.pk, "label": personal_data.label}),
                        content_type='application/json')



from django.conf.urls.defaults import patterns, url
from django.views.generic import DetailView

from views import ReportCreateView
from models import Report


urlpatterns = patterns('',
    # Create Report
    url(r'^(?P<content_type>\d+)/(?P<object_id>\d+)/?$', ReportCreateView.as_view(), name='report'),
    # Detail Report
    url(r'^(?P<pk>\d+)/?$', DetailView.as_view(model=Report), name='report_detail'),
)



from django import forms
from django.forms import formsets, ImageField, FileInput
from django.forms.formsets import formset_factory
from elections.models import Candidate, Link, BackgroundCandidate, PersonalDataCandidate, Answer

class CandidateForm(forms.ModelForm):
    class Meta:
        model = Candidate
        exclude = ('election', 'answers', 'personal_data', 'background', 'photo', 'slug', 'has_answered')
    class Media:
        js = ('js/jquery.slug.js', 'js/jquery.formset.js', )
        css = {
            'all': ('css/wizard-forms.css',)
            }
    
class CandidatePhotoForm(forms.ModelForm):
    photo = ImageField(widget=FileInput)
    def __init__(self, candidate, *args, **kwargs):
        self.candidate = candidate
        super(CandidatePhotoForm, self).__init__(*args, **kwargs)

    class Meta:
        model = Candidate
        fields = ('photo',)
    
    def save(self, commit=True):
        self.candidate.photo = self.cleaned_data['photo']
        if commit:
            self.candidate.save()
        return self.candidate

class CandidateUpdateForm(forms.ModelForm):
    class Meta:
        model = Candidate
        exclude = ('slug', 'election', 'answers', 'personal_data', 'background', 'has_answered')
    class Media:
        js = ('js/jquery.formset.js', )


class CandidateLinkForm(forms.ModelForm):
    class Meta:
        model = Link
        exclude = ('candidate', )


class BackgroundCandidateForm(forms.ModelForm):
    class Meta:
        model = BackgroundCandidate
        exclude = ('candidate', )


class PersonalDataCandidateForm(forms.ModelForm):
    class Meta:
        model = PersonalDataCandidate
        exclude = ('candidate', )


#CandidateLinkFormset = formsets.formset_factory(CandidateLinkForm, extra=1)



from django.test import TestCase
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.template import Template, Context

from models import Report
from forms import ReportForm


class ReportModelTest(TestCase):
    def test_create(self):
        user, created = User.objects.get_or_create(username='asd')
        report = Report.objects.create(content_object=user, reason='reason lorem ipsum')
        self.assertEquals(report.content_object, user)
        self.assertEquals(report.reason, 'reason lorem ipsum')

    def test_create_with_owner(self):
        user, created = User.objects.get_or_create(username='asd')
        report = Report.objects.create(content_object=user, reason='reason lorem ipsum', owner=user)
        self.assertEquals(report.content_object, user)
        self.assertEquals(report.reason, 'reason lorem ipsum')
        self.assertEquals(report.owner, user)


class ReportCreateViewTest(TestCase):
    '''
    '''
    def test_get_create_report(self):
        user = User.objects.create_user(username='foo', password='bar', email='foo@bar.com')
        content_type = ContentType.objects.get_for_model(User)
        url = reverse('report', kwargs={'object_id': user.pk, 'content_type': content_type.pk})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue('report_object' in response.context)
        self.assertTrue(isinstance(response.context['report_object'], User))
        self.assertEquals(response.context['report_object'], user)
        self.assertTrue('form' in response.context)
        self.assertTrue(isinstance(response.context['form'], ReportForm))
        self.assertTrue('reason' in response.context['form'].fields)

    def test_post_create_report(self):
        user = User.objects.create_user(username='foo', password='bar', email='foo@bar.com')
        content_type = ContentType.objects.get_for_model(User)
        url = reverse('report', kwargs={'object_id': user.pk, 'content_type': content_type.pk})
        response = self.client.post(url, {'reason': 'lorem ipsum dolor sit amet'}, follow=True)
        self.assertEqual(response.status_code, 200)
        report = Report.objects.get(pk=1)
        self.assertEquals(report.reason, 'lorem ipsum dolor sit amet')
    
    def test_get_create_report_without_content_type(self):
        user = User.objects.create_user(username='foo', password='bar', email='foo@bar.com')
        content_type = ContentType.objects.get_for_model(User)
        url = reverse('report', kwargs={'object_id': user.pk, 'content_type': 1000})
        response = self.client.post(url, params={'reason': 'lorem ipsum dolor sit amet'}, follow=True)
        self.assertEqual(response.status_code, 404)


class ReportTemplatetagTest(TestCase):
    def test_get_url_for(self):
        user = User.objects.create_user(username='foo', password='bar', email='foo@bar.com')
        content_type = ContentType.objects.get_for_model(User)
        result = Template(
            '{% load report_tags %}'
            '{% report_url user %}'
        ).render(Context({
            'user': user
        }))
        self.assertEquals(result, reverse('report', kwargs={'object_id': user.pk, 'content_type': content_type.pk}))

class ReportURLTest(TestCase):
    
    def test_create(self):
        url = reverse('report', kwargs={'object_id': 1, 'content_type': 3})
        self.assertEquals(url, '/report/3/1')



import os
from django.conf import settings
from django.test import TestCase
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.db import IntegrityError

from elections.models import Link, Candidate, Election
from django.core.exceptions import ValidationError
#from elections.forms import CandidateLinkFormset

dirname = os.path.dirname(os.path.abspath(__file__))

class LinkModelTest(TestCase):
    def setUp(self):
        self.user, created = User.objects.get_or_create(username='joe')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=self.user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion')

        self.candidate, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                                                  election=self.election)

    def test_create_link(self):
        link, created = Link.objects.get_or_create(
                                                    name='Google',
                                                    url='http://www.google.com',
                                                    candidate=self.candidate)
        self.assertTrue(created)
        self.assertEqual(link.name, 'Google')
        self.assertEqual(link.url, 'http://www.google.com')
        self.assertEqual(link.candidate, self.candidate)

    def test_does_not_create_an_empty_link(self):
        link = Link(name="", url="", candidate=self.candidate)
        try:
            link.full_clean()
            self.fail('The link name can not be empty')
        except ValidationError as e:
            expected_error = {'name':[u'This field cannot be blank.'], 'url':[u'This field cannot be blank.']}
            self.assertEqual(e.message_dict,expected_error)

        
        link = Link(name="", url="http://www.google.com", candidate=self.candidate)
        try:
            link.full_clean()
            self.fail('The link name can not be empty')
        except ValidationError as e:
            expected_error = {'name':[u'This field cannot be blank.']}
            self.assertEqual(e.message_dict,expected_error)


        link = Link(name="Twitter", url="", candidate=self.candidate)
        try:
            link.full_clean()
            self.fail('The url not can be empty')
        except ValidationError as e:
            expected_error = {'url':[u'This field cannot be blank.']}
            self.assertEqual(e.message_dict,expected_error)

    def test_http_prefixes(self):
        link, created = Link.objects.get_or_create(
                                                    name = 'Twitter',
                                                    url = 'www.twitter.com',
                                                    candidate = self.candidate)

        link2, created = Link.objects.get_or_create(
                                                    name = 'Twitter',
                                                    url = 'http://www.twitter.com',
                                                    candidate = self.candidate)
        self.assertTrue(created)
        self.assertEqual(link.http_prefix, 'http://www.twitter.com')
        self.assertEqual(link2.http_prefix, 'http://www.twitter.com')

    def test_css_classess(self):
        link, created = Link.objects.get_or_create(
                                                    name='Facebook',
                                                    url='http://www.facebook.com',
                                                    candidate=self.candidate)
        self.assertTrue(created)
        self.assertEqual(link.css_class, 'facebook')

        link, created = Link.objects.get_or_create(
                                                    name='Twitter',
                                                    url='http://www.twitter.com',
                                                    candidate=self.candidate)
        self.assertTrue(created)
        self.assertEqual(link.css_class, 'twitter')


class AsyncDeleteLinkTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=self.user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion')

        self.candidate, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                                                  election=self.election)
        self.link, created = Link.objects.get_or_create(name='Google',
                                                        url='http://www.google.com',
                                                        candidate=self.candidate)


    def test_post_with_login(self):
        self.client.login(username='joe', password='doe')

        request = self.client.post(reverse('async_delete_link',
                                kwargs={'link_pk': self.link.pk}),
                                        {})
        self.assertEquals(request.status_code, 200)

    def test_post_without_login(self):
        request = self.client.post(reverse('async_delete_link',
                                kwargs={'link_pk': self.link.pk}),
                                        {})
        self.assertEquals(request.status_code, 302)


    def test_get_405(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('async_delete_link',
                                kwargs={'link_pk': self.link.pk}))
        self.assertEquals(request.status_code, 405)

    def test_post_with_stranger_candidate(self):
        user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        election2, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user2,
                                                           slug='barbaz2',
                                                           description='esta es una descripcion')

        candidate2, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                                                  election=election2)
        link2, created = Link.objects.get_or_create(name='Google',
                                                    url='http://www.google.com',
                                                    candidate=candidate2)


        self.client.login(username='joe', password='doe')
        request = self.client.post(reverse('async_delete_link',
                                kwargs={'link_pk': link2.pk}))

        self.assertEquals(request.status_code, 404)

    def test_post_success(self):
        self.client.login(username='joe', password='doe')
        temp_pk = self.link.pk
        request = self.client.post(reverse('async_delete_link',
                                kwargs={'link_pk': self.link.pk}),
                                        {})

        self.assertEquals(request.status_code, 200)
        self.assertEquals(request.content, '{"result": "OK"}')

        self.assertRaises(Link.DoesNotExist, Link.objects.get, pk=temp_pk)


from django.conf.urls.defaults import *
from django.views.generic.simple import direct_to_template, redirect_to

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
from tastypie.api import Api


admin.autodiscover()


urlpatterns = patterns('',
    url(r'^admin/doc/', include('django.contrib.admindocs.urls')),

    (r'^grappelli/', include('grappelli.urls')),
    url(r'^admin/', include(admin.site.urls)),

    (r'^profiles/', include('profiles.urls')),

    # django-registration urls, maps common registration urls to the ones in django.contrib.auth
    url(r'^accounts/', include('registration.urls')),

    url(r'^accounts/password_reset/?$','django.contrib.auth.views.password_reset'),

    url(r'^report/', include('report_objects.urls')),

    (r'^api/', include('elections.api_urls')),

    url(r'^', include('elections.urls')),



    

    # url(r'^index/?$', direct_to_template, {'template': 'index.html'}), # DESCOMENTAR CUANDO SE DEFINA UN INDEX

)

from django.conf import settings
if settings.DEBUG:
    from django.conf.urls.static import static

    urlpatterns += static( settings.USER_FILES, document_root=settings.STATIC_ROOT )
    urlpatterns += patterns('',
        url(r'^media/(?P<path>.*)$', 'django.views.static.serve', {
            'document_root': settings.MEDIA_ROOT,
        }),
   )



from django.test import TestCase
from django.contrib.auth import authenticate, login
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test.client import Client

from django.db import IntegrityError

from elections.models import Candidate, Election, Category, Question, Answer
from elections.forms import QuestionForm, CategoryForm, ElectionForm, CategoryUpdateForm
from django.core.exceptions import ValidationError

class CategoryModelTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')

    def test_create_category(self):
        category = Category.objects.create(name='FooCat', election=self.election)

        self.assertEqual(category.name, 'FooCat')
        self.assertEqual(category.slug, 'foocat')
        self.assertEqual(category.election, self.election)
        self.assertEqual(category.order, 1)

    def test_create_category_with_same_name(self):
        category = Category.objects.get_or_create(name='FooCat', election=self.election)
        self.assertRaises(IntegrityError, Category.objects.create,
                          name='FooCat', election=self.election)

    def test_update_category(self):
        category = Category.objects.create(name='FooCat', election=self.election)
        new_category_name = 'FooBar'
        category.name = new_category_name
        category.save()
        updated_category = Category.objects.get(slug='foocat', election=self.election)
        self.assertEqual(updated_category.name, new_category_name)

    def test_category_name_cannot_be_empty(self):
        category = Category.objects.create(name='', election=self.election)
        try:
            category.full_clean()
            self.fail('The category name can be empty')
        except ValidationError:
            pass

    def test_ordered_categories(self):
        [category.delete() for category in self.election.category_set.all()]
        category2 = Category.objects.create(name=u'seccond election', election=self.election, order=2)
        category1 = Category.objects.create(name=u'first election', election=self.election, order=1)

        categories = Category.objects.filter(election=self.election)
        self.assertEquals(categories[0].name, u'first election')
        self.assertEquals(categories[1].name, u'seccond election')

class CategoryCreateViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election = Election.objects.create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')


    def test_create_category_by_user_without_login(self):
        request = self.client.get(reverse('category_create',
                                    kwargs={'election_slug': self.election.slug}))
        self.assertEquals(request.status_code, 302)

    def test_create_category_by_user_success(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('category_create',
                                    kwargs={'election_slug': self.election.slug}))

        self.assertEqual(request.status_code, 200)
        self.assertTrue('form' in request.context)
        self.assertTrue(isinstance(request.context['form'], CategoryForm))
        self.assertTrue('election' in request.context)
        self.assertTrue(isinstance(request.context['election'], Election))

    def test_post_category_create_with_same_slug(self):
        category = Category.objects.create(name="Bar1", election=self.election)

        self.client.login(username='joe', password='doe')
        params = {'name': 'Bar1'}
        response = self.client.post(reverse('category_create',
                                        kwargs={'election_slug': self.election.slug}),
                                    params, follow=True)

        self.assertEquals(response.status_code, 200)
        self.assertFormError(response, 'form', 'slug', 'Ya tienes una categoria con ese slug.')


    def test_post_category_create_without_login(self):
        params = {'name': 'BarBaz'}
        response = self.client.post(reverse('category_create',
                                        kwargs={'election_slug': self.election.slug}),
                                    params)

        self.assertEquals(response.status_code, 302)

    def test_get_category_create_with_login_stranger_election(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('category_create',
                                    kwargs={'election_slug': 'strager_election_slug'}))
        self.assertEquals(response.status_code, 404)

    def test_post_category_create_with_login_stranger_election(self):
        self.client.login(username='joe', password='doe')

        params = {'name': 'Bar'}
        response = self.client.post(reverse('category_create',
                                        kwargs={'election_slug': 'strager_election_slug'}),
                                    params)
        self.assertEquals(response.status_code, 404)

    def test_post_category_create_logged(self):
        self.client.login(username='joe', password='doe')

        params = {'name': 'BarBaz'}
        response = self.client.post(reverse('category_create',
                                        kwargs={'election_slug': self.election.slug}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)
        qs = Category.objects.filter(name='BarBaz')
        self.assertEquals(qs.count(), 1)
        category = qs.get()
        self.assertEquals(category.name, 'BarBaz')
        self.assertEquals(category.slug, 'barbaz')
        self.assertEquals(category.election, self.election)

        self.assertRedirects(response, reverse('category_create',
                                               kwargs={'election_slug': self.election.slug}))

    def test_cannot_create_a_category_with_an_empty_name(self):
        self.client.login(username='joe', password='doe')
        params = {'name': ''}
        response = self.client.post(reverse('category_create',
                                        kwargs={'election_slug': self.election.slug}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)
        self.assertFormError(response, 'form', 'name', u'This field is required.')


class CategoryUpdateViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election = Election.objects.create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz',
                                                            published=True)
        self.category = Category.objects.create(name="Bar1", slug="bar", election=self.election)


    def test_update_category_by_user_without_login(self):
        request = self.client.get(reverse('category_update',
                                    kwargs={'slug': self.category.slug,
                                            'election_slug': self.election.slug}))
        self.assertEquals(request.status_code, 302)

    def test_update_category_by_user_success(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('category_update',
                                    kwargs={'slug': self.category.slug,
                                            'election_slug': self.election.slug}))

        # print request.context
        self.assertEqual(request.status_code, 200)

        self.assertTrue('form' in request.context)
        self.assertTrue(isinstance(request.context['form'], CategoryUpdateForm))
        self.assertEqual(request.context['form'].initial['name'], self.category.name)
        self.assertTrue('election' in request.context)
        self.assertTrue(isinstance(request.context['election'], Election))


    def test_post_category_update_without_login(self):
        params = {'name': 'BarBaz'}
        response = self.client.post(reverse('category_update',
                                        kwargs={'slug': self.election.slug,
                                                'election_slug': self.election.slug}),
                                    params)

        self.assertEquals(response.status_code, 302)

    def test_get_category_update_with_login_stranger_election(self):
        user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        election2 = Election.objects.create(name='BarBaz',
                                            owner=user2,
                                            slug='barbaz2')
        category2 = Category.objects.create(name="Bar1", slug="bar2", election=election2)

        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('category_update',
                                    kwargs={'slug': category2.slug,
                                            'election_slug': election2.slug}))
        self.assertEquals(response.status_code, 404)

    def test_post_category_update_with_login_stranger_election(self):
        user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        election2 = Election.objects.create(name='BarBaz',
                                            owner=user2,
                                            slug='barbaz2')
        category2 = Category.objects.create(name="Bar1", slug="bar2", election=election2)

        self.client.login(username='joe', password='doe')

        params = {'name': 'Bar'}
        response = self.client.post(reverse('category_update',
                                    kwargs={'slug': category2.slug,
                                            'election_slug': election2.slug}),
                                    params)
        self.assertEquals(response.status_code, 404)

    def test_post_category_update_logged(self):
        self.client.login(username='joe', password='doe')

        params = {'name': 'Bar2'}
        response = self.client.post(reverse('category_update',
                                        kwargs={'slug': self.category.slug,
                                                'election_slug': self.election.slug}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)
        qs = Category.objects.filter(slug=self.category.slug, election=self.election)
        self.assertEquals(qs.count(), 1)
        category = qs.get()
        self.assertEquals(category.name, params['name'])

        self.assertRedirects(response, reverse('election_detail',
                                               kwargs={'username':self.user.username, 'slug': self.election.slug}))


class AsyncDeleteCategoryTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=self.user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion')

        self.category = Category.objects.create(name="Bar1", slug="bar", election=self.election)


    def test_post_with_login(self):
        self.client.login(username='joe', password='doe')

        request = self.client.post(reverse('async_delete_category',
                                kwargs={'category_pk': self.category.pk}),
                                        {})
        self.assertEquals(request.status_code, 200)

    def test_post_without_login(self):
        request = self.client.post(reverse('async_delete_category',
                                kwargs={'category_pk': self.category.pk}),
                                        {})
        self.assertEquals(request.status_code, 302)


    def test_get_405(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('async_delete_category',
                                kwargs={'category_pk': self.category.pk}))
        self.assertEquals(request.status_code, 405)

    def test_post_with_stranger_candidate(self):
        user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        election2, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user2,
                                                           slug='barbaz2',
                                                           description='esta es una descripcion')

        category2 = Category.objects.create(name="Bar1", slug="bar2", election=election2)

        self.client.login(username='joe', password='doe')
        request = self.client.post(reverse('async_delete_category',
                                kwargs={'category_pk': category2.pk}))

        self.assertEquals(request.status_code, 404)

    def test_post_success(self):
        self.client.login(username='joe', password='doe')
        temp_pk = self.category.pk
        request = self.client.post(reverse('async_delete_category',
                                kwargs={'category_pk': self.category.pk}),
                                        {})

        self.assertEquals(request.status_code, 200)
        self.assertEquals(request.content, '{"result": "OK"}')

        self.assertRaises(Category.DoesNotExist, Category.objects.get, pk=temp_pk)


class AsyncCreateCategoryViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')

        self.category = Category.objects.create(name="Bar1", slug="bar", election=self.election)

        self.user2 = User.objects.create_user(username='johnny', password='doe', email='johnny@doe.cl')

        self.election2, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user2,
                                                            slug='barbaz')
        self.category2 = Category.objects.create(name="Bar2", slug="bar", election=self.election2)

    def test_get_async_create_category_with_login(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('async_create_category',
                                    kwargs={'election_pk': self.election.pk}))
        self.assertEqual(response.status_code, 405)

    def test_get_async_create_category_without_login(self):
        response = self.client.get(reverse('async_create_category',
                                    kwargs={'election_pk': self.election.pk}))
        self.assertEqual(response.status_code, 302)


    def test_post_async_create_category_without_login(self):
        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_category',
                                    kwargs={'election_pk': self.election.pk}),
                                    params)

        self.assertEquals(response.status_code, 302)

    def test_post_async_create_category_with_login_stranger_category(self):
        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_category',
                                    kwargs={'election_pk': self.election2.pk}),
                                    params)
        self.assertEquals(response.status_code, 404)

    def test_post_async_create_category_logged(self):
        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_category',
                                    kwargs={'election_pk': self.election.pk}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)

        categories = self.election.category_set.all()
        category_names = [ category.name for category in categories]

        self.assertTrue(params['value'] in category_names)
        category = self.election.category_set.get(name=params['value'])
        self.assertEquals(response.content, '{"pk": %d, "name": "%s"}' % (category.pk, params['value']))

    def test_post_async_create_category_with_an_empty_name(self):
        self.client.login(username='joe', password='doe')

        params = {'value': ''}
        response = self.client.post(reverse('async_create_category',
                                    kwargs={'election_pk': self.election.pk}),
                                    params,
                                    follow=True)

        #412 means error
        self.assertEquals(response.status_code, 412)



# coding= utf-8
from django.core.management.base import BaseCommand, CommandError
from candideitorg.models import Election

class Command(BaseCommand):
    args = ''

    def handle(self, *args, **options):
        Election.fetch_all_from_api()


from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.forms import formsets
from django.http import HttpResponse, Http404
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.template import RequestContext
from django.template.context import RequestContext
from django.utils import simplejson as json
from django.utils.decorators import method_decorator
from django.views.decorators.http import require_http_methods
from django.views.generic import CreateView, DetailView, UpdateView

# Import forms
from elections.forms import BackgroundCategoryForm, BackgroundForm, BackgroundCandidateForm

# Import models
from elections.models import Background, BackgroundCategory, Candidate


# Background Views
class BackgroundCreateView(CreateView):
    model = Background
    form_class = BackgroundForm

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(BackgroundCreateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(BackgroundCreateView, self).get_context_data(**kwargs)
        context['background_category'] = get_object_or_404(BackgroundCategory, pk=self.kwargs['background_category_pk'], election__owner=self.request.user)
        return context

    def get_success_url(self):
        return reverse('background_category_create', kwargs={'election_slug': self.object.category.election.slug})

    def form_valid(self, form):
        self.object = form.save(commit=False)
        background_category = get_object_or_404(BackgroundCategory, pk=self.kwargs['background_category_pk'], election__owner=self.request.user)
        self.object.category = background_category
        return super(BackgroundCreateView, self).form_valid(form)


@login_required
@require_http_methods(['GET', 'POST'])
def background_candidate_create(request, candidate_pk, background_pk):
    candidate = get_object_or_404(Candidate, pk=candidate_pk, election__owner=request.user)
    background = get_object_or_404(Background, pk=background_pk, category__election__owner=request.user)

    if request.POST:
        value = request.POST.get('value', None)
        candidate.add_background(background, value)
        return HttpResponse(json.dumps({"value": value}),
                            content_type='application/json')
    form = BackgroundCandidateForm()
    return render_to_response(\
            'elections/background_associate.html',
            {'candidate': candidate, 'background': background, 'form':form},
            context_instance=RequestContext(request))


@login_required
@require_http_methods(['POST'])
def async_create_background(request, background_category_pk):
    background_category = get_object_or_404(BackgroundCategory, pk=background_category_pk, election__owner=request.user)

    value = request.POST.get('value', None)
    background = Background(name=value, category=background_category)
    background.save()
    return HttpResponse(json.dumps({"pk": background.pk, "name": background.name}),
                        content_type='application/json')


@login_required
@require_http_methods(['POST'])
def async_delete_background(request, background_pk):
    background = get_object_or_404(Background, pk=background_pk, category__election__owner=request.user)
    background.delete()
    json_dictionary = {"result":"OK"}
    return HttpResponse(json.dumps(json_dictionary),content_type='application/json')



from tests import *

from answer import *
from api import *
from background import *
from background_category import *
from candidate import *
from candidate_answer import *
from category import *
from election import *
from link import *
from medianaranja import *
from personal_data import *
from question import *
from election_embeded import *
from election_loader import *
from settings_variables import *
from user import *
from api_v2 import *
from information_source import *


# encoding=UTF-8
from django.db import models
from django.contrib.auth.models import User
from django.db.models.signals import post_save
from tastypie.models import create_api_key

# Create your models here.

class Profile(models.Model):
    name = models.CharField(u'Nombre', max_length=255)
    biography = models.TextField(u'Cuéntanos un poco sobre tí...')

    user = models.ForeignKey('auth.User')
    
    @models.permalink
    def get_absolute_url(self):
        return ('profiles_profile_detail', (), { 'username': self.user.username })


def create_user_profile(sender, instance, created, **kwargs):
    if created:
        Profile.objects.create(user=instance, name=instance.username)

models.signals.post_save.connect(create_api_key, sender=User)
post_save.connect(create_user_profile, sender=User)



# -*- coding: utf-8 -*-
from django import template
from django.utils.translation import ugettext as _

register = template.Library()

@register.simple_tag
def answer_for_candidate_and_question(candidate, question):
    '''
    Returns the answer for the given candidate and question pair.

    >> {% answer_for_candidate_and_question candidate question %}
    "answer"
    '''

    try:
        return question.answer_set.get(candidate=candidate).caption
    except:
        pass
    return _(u"Aún no hay respuesta")

@register.inclusion_tag('candideitorg/information_source.html')
def get_information_source(candidate, question):
    '''
    Returns the information source for a candidate referred to a certain question.

    >> {% get_information_source candidate question %}
    "answer"
    '''
    try:
        information_source = question.informationsource_set.get(candidate=candidate)
        return {'information_source':information_source}
    except:
        return {'information_source':None}

@register.simple_tag
def relation_personal_data_candidate(candidate, personaldata):
    try:
        return personaldata.personaldatacandidate_set.get(candidate=candidate).value
    except:
        return _('no message')

@register.simple_tag
def relation_background_candidate(candidate, background):
    try:
        return background.backgroundcandidate_set.get(candidate=candidate).value
    except:
        return _('no message')


from django.contrib import admin
from models import *


def admin_thumbnail(self):
    return u'<img src="%s" />' % (self.image.url)

admin_thumbnail.short_description = 'Thumbnail'
admin_thumbnail.allow_tags = True


class CandidateInline(admin.TabularInline):
    model = Candidate
    extra = 1
    exclude = ('answers',)


class PersonalDataInline(admin.TabularInline):
    model = PersonalData
    extra = 1


class BackgroundCategoryInline(admin.TabularInline):
    model = BackgroundCategory
    extra = 1


class CategoryInline(admin.TabularInline):
    model = Category
    sortable_field_name = 'order'
    extra = 0


class ElectionAdmin(admin.ModelAdmin):
    list_display = ('name', 'owner', 'description', 'slug', admin_thumbnail)
    search_fields = ['name', 'description']
    inlines = [CandidateInline, PersonalDataInline, BackgroundCategoryInline, CategoryInline]


admin.site.register(Election, ElectionAdmin)


class LinkInline(admin.TabularInline):
    model = Link
    extra = 1


class CandidateAdmin(admin.ModelAdmin):
    list_display = ('name', 'created_at', 'slug', admin_thumbnail, 'election')
    list_filter = ('election__name',)
    search_fields = ['election__name', 'first_name', 'last_name']
    exclude = ('answers',)
    inlines = [LinkInline]


admin.site.register(Candidate, CandidateAdmin)


class QuestionInLine(admin.StackedInline):
    model = Question
    extra = 1


class CategoryAdmin(admin.ModelAdmin):
    list_filter = ('election__name',)
    inlines = [QuestionInLine]
    search_fields = ['name', 'election__name']

admin.site.register(Category, CategoryAdmin)


class AnswerInLine(admin.StackedInline):
    model = Answer
    extra = 1


class QuestionAdmin(admin.ModelAdmin):
    list_filter = ('category__election__name',)
    search_fields = ['question', 'category__election__name']
    inlines = [AnswerInLine]


admin.site.register(Question, QuestionAdmin)
admin.site.register(PersonalDataCandidate)
admin.site.register(BackgroundCandidate)
admin.site.register(Background)
admin.site.register(BackgroundCategory)

# admin de registro de medianaranja
class VisitorAnswerInLine(admin.StackedInline):
    model = VisitorAnswer
    extra = 0
class CategoryScoreInLine(admin.StackedInline):
    model = CategoryScore
    extra = 0
class VisitorScoreInLine(admin.StackedInline):
    model = VisitorScore
    extra = 0
class VisitorScoreAdmin(admin.ModelAdmin):
    model = VisitorScore
    extra = 0
    inlines = [CategoryScoreInLine]
class VisitorAdmin(admin.ModelAdmin):
    model=Visitor
    extra=0
    inlines = [VisitorScoreInLine]
admin.site.register(Visitor,VisitorAdmin)
admin.site.register(VisitorScore,VisitorScoreAdmin)



from django.test import TestCase
from django.contrib.auth import authenticate, login
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.utils import simplejson
from elections.models import Election, Category, Question
from elections.forms import QuestionForm
from django.core.exceptions import ValidationError

class QuestionModelTest(TestCase):
    def test_create_question(self):
        user, created = User.objects.get_or_create(username='joe')
        election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=user,
                                                            slug='barbaz')
        category, created = Category.objects.get_or_create(name='FooCat',
                                                           slug='foocat',
                                                            election=election)
        question = Question.objects.create(question='Foo', category=category)
        self.assertEquals(question.question, 'Foo')
        self.assertEquals(question.category, category)


    def test_cannot_create_empty_question(self):
        user, created = User.objects.get_or_create(username='joe')
        election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=user,
                                                            slug='barbaz')
        category, created = Category.objects.get_or_create(name='FooCat',
                                                           slug='foocat',
                                                            election=election)
        question = Question(question='', category=category)

        with self.assertRaises(ValidationError) as e:
            question.full_clean()
            expected_error = {'question':[u'This field cannot be blank.']}
            self.assertEqual(e.message_dict,expected_error)
            

class QuestionCreateViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election = Election.objects.create(name='BarBaz',
                                                owner=self.user,
                                                slug='barbaz')
        self.category, created = Category.objects.get_or_create(name='FooCat',
                                                                slug='foocat',
                                                                election=self.election)


    def test_create_question_by_user_without_login(self):
        request = self.client.get(reverse('question_create',
                                    kwargs={'election_slug': self.election.slug}))
        self.assertEquals(request.status_code, 302)

    def test_create_question_by_user_success(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('question_create',
                                    kwargs={'election_slug': self.election.slug}))

        self.assertEqual(request.status_code, 200)
        self.assertTrue('form' in request.context)
        self.assertTrue(isinstance(request.context['form'], QuestionForm))
        self.assertTrue('election' in request.context)
        self.assertTrue(isinstance(request.context['election'], Election))
    
    def test_step_five_template_rendered(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('question_create',
                                    kwargs={'election_slug': self.election.slug}))
        
        self.assertTemplateUsed(response, 'elections/wizard/step_five.html')

    def test_post_question_create_without_login(self):
        params = {'question': 'BarBaz', 'new_category': 'NewCategory', 'category': None}
        response = self.client.post(reverse('question_create',
                                    kwargs={'election_slug': self.election.slug}), params)

        self.assertEquals(response.status_code, 302)

    def test_get_question_create_with_login_stranger_election(self):
        user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        election2 = Election.objects.create(name='BarBaz',
                                            owner=user2,
                                            slug='barbaz2')

        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('question_create',
                                    kwargs={'election_slug': election2.slug}))
        self.assertEquals(response.status_code, 404)

    def test_post_question_create_with_login_stranger_election(self):
        user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        election2 = Election.objects.create(name='BarBaz',
                                            owner=user2,
                                            slug='barbaz2')

        self.client.login(username='joe', password='doe')

        params = {'question': 'BarBaz', 'new_category': 'NewCategory', 'category': None}
        response = self.client.post(reverse('question_create',
                                    kwargs={'election_slug': election2.slug}), params)
        self.assertEquals(response.status_code, 404)

    def test_post_question_create_logged_into_new_category(self):
        self.client.login(username='joe', password='doe')

        params = {'question': 'BarBaz', 'new_category': 'NewCategory'}
        response = self.client.post(reverse('question_create',
                                        kwargs={'election_slug': self.election.slug}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)
        qs = Question.objects.filter(question=params['question'])
        self.assertEquals(qs.count(), 1)
        question = qs.get()
        self.assertEquals(question.question, params['question'])

        category = Category.objects.get(name=params['new_category']);
        self.assertEquals(question.category, category)

        self.assertRedirects(response, reverse('question_create',
                                               kwargs={'election_slug': self.election.slug}))

    def test_post_question_create_logged_into_existing_category(self):
        self.client.login(username='joe', password='doe')
        category, created = Category.objects.get_or_create(name='FooCat', election=self.election)

        params = {'question': 'BarBaz', 'new_category': '', 'category': category.pk}
        response = self.client.post(reverse('question_create',
                                    kwargs={'election_slug': self.election.slug}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)
        qs = Question.objects.filter(question=params['question'])
        self.assertEquals(qs.count(), 1)
        question = qs.get()
        self.assertEquals(question.question, params['question'])

        self.assertEquals(question.category.pk, params['category'])

        self.assertRedirects(response, reverse('question_create',
                                               kwargs={'election_slug': self.election.slug}))

    def test_post_question_create_logged_into_nothing(self):
        self.client.login(username='joe', password='doe')
        category, created = Category.objects.get_or_create(name='FooCat', election=self.election)

        params = {'question': 'BarBaz'}
        response = self.client.post(reverse('question_create',
                                    kwargs={'election_slug': self.election.slug}),
                                    params)

        self.assertEquals(response.status_code, 200)
        self.assertFormError(response, 'form', 'category', 'Este campo es obligatorio.')


class AsyncDeleteQuestionTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=self.user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion')

        self.category = Category.objects.create(name="Bar1", slug="bar", election=self.election)
        self.question = Question.objects.create(question='Fooo', category=self.category)

    def test_post_with_login(self):
        self.client.login(username='joe', password='doe')

        request = self.client.post(reverse('async_delete_question',
                                kwargs={'question_pk': self.question.pk}),
                                        {})
        self.assertEquals(request.status_code, 200)

    def test_post_without_login(self):
        request = self.client.post(reverse('async_delete_question',
                                kwargs={'question_pk': self.question.pk}),
                                        {})
        self.assertEquals(request.status_code, 302)


    def test_get_405(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('async_delete_question',
                                kwargs={'question_pk': self.question.pk}))
        self.assertEquals(request.status_code, 405)

    def test_post_with_stranger_candidate(self):
        user2 = User.objects.create_user(username='doe', password='doe', email='joe@doe.cl')
        election2, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user2,
                                                           slug='barbaz2',
                                                           description='esta es una descripcion')

        category2 = Category.objects.create(name="Bar1", slug="bar2", election=election2)
        question2 = Question.objects.create(question='Foo', category=category2)

        self.client.login(username='joe', password='doe')
        request = self.client.post(reverse('async_delete_question',
                                kwargs={'question_pk': question2.pk}))

        self.assertEquals(request.status_code, 404)

    def test_post_success(self):
        self.client.login(username='joe', password='doe')
        temp_pk = self.question.pk
        request = self.client.post(reverse('async_delete_question',
                                kwargs={'question_pk': self.question.pk}),
                                        {})

        self.assertEquals(request.status_code, 200)
        self.assertEquals(request.content, '{"result": "OK"}')

        self.assertRaises(Question.DoesNotExist, Question.objects.get, pk=temp_pk)

class AsyncCreateQuestionViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')

        self.category = Category.objects.create(name="Bar1", slug="bar", election=self.election)

        self.user2 = User.objects.create_user(username='johnny', password='doe', email='johnny@doe.cl')

        self.election2, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user2,
                                                            slug='barbaz')
        self.category2 = Category.objects.create(name="Bar2", slug="bar", election=self.election2)

    def test_get_async_create_question_with_login(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('async_create_question',
                                    kwargs={'category_pk': self.category.pk}))
        self.assertEqual(response.status_code, 405)

    def test_get_async_create_question_without_login(self):
        response = self.client.get(reverse('async_create_question',
                                    kwargs={'category_pk': self.category.pk}))
        self.assertEqual(response.status_code, 302)


    def test_post_async_create_question_without_login(self):
        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_question',
                                    kwargs={'category_pk': self.category.pk}),
                                    params)

        self.assertEquals(response.status_code, 302)

    def test_post_async_create_question_with_login_stranger_category(self):
        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_question',
                                    kwargs={'category_pk': self.category2.pk}),
                                    params)
        self.assertEquals(response.status_code, 404)

    def test_post_async_create_question_logged(self):
        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_question',
                                    kwargs={'category_pk': self.category.pk}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)

        questions = self.category.question_set.all()
        question_names = [ question.question for question in questions]

        self.assertTrue(params['value'] in question_names)
        question = self.category.question_set.get(question=params['value'])
        self.assertEquals(response.content, '{"pk": %d, "question": "%s"}' % (question.pk, params['value']))


    def test_post_creaete_empty_question(self):
        self.client.login(username='joe', password='doe')

        params = {'value': ''}
        url = reverse('async_create_question',kwargs={'category_pk': self.category.pk})
        response = self.client.post(url, params, HTTP_X_REQUESTED_WITH='XMLHttpRequest')

        questions_number = Question.objects.filter(category=self.category).count()
        self.assertEqual(questions_number, 0)



        self.assertEquals(response.status_code, 200)
        self.assertTrue('error' in simplejson.loads(response.content))


        expected_error = {u'error':{u'question':[u'This field cannot be blank.']}}

        self.assertEqual(simplejson.loads(response.content), expected_error)

        self.assertEqual(response['Content-Type'], 'application/json')



        questions = self.category.question_set.all()
        question_names = [ question.question for question in questions]



from django import template
from django.core.urlresolvers import reverse
from django.contrib.contenttypes.models import ContentType

register = template.Library()


@register.simple_tag
def report_url(obj):
    '''
    Returns the url for reporting any object.

    >> {% report_url user %}
    "/report/1/2"
    '''
    content_type = ContentType.objects.get_for_model(type(obj))
    return reverse('report', kwargs={'object_id': obj.pk, 'content_type': content_type.pk})



# -*- coding: utf-8 -*-
from django import forms
from django.forms.util import ErrorList
from django.forms import formsets, ImageField, FileInput
from django.forms.formsets import formset_factory
from elections.models import Category, Election, PersonalData,\
                BackgroundCategory, Background, Question, Answer,\
                PersonalDataCandidate, BackgroundCandidate


class ElectionForm(forms.ModelForm):
    class Meta:
        model = Election
        exclude = ('owner', 'slug')
    class Media:
        js = ('js/jquery.slug.js', )
        css = { 'all': ('css/wizard-forms.css',) }


class ElectionUpdateForm(forms.ModelForm):
    class Meta:
        model = Election
        exclude = ('owner', 'slug', 'published')


class ElectionStyleUpdateForm(forms.ModelForm):
    class Meta:
        model = Election
        exclude = ('owner', 'slug', 'name', 'description', 'information_source','logo','created_at', 'updated_at','date', 'published')



        
class ElectionLogoUpdateForm(forms.ModelForm):
    logo = ImageField(widget=FileInput)
    def __init__(self, election, *args, **kwargs):
        self.election = election
        super(ElectionLogoUpdateForm, self).__init__(*args, **kwargs)


    class Meta:
        model = Election
        fields = ('logo',)

    def save(self, commit=True):
        self.election.logo = self.cleaned_data['logo']
        if commit:
            self.election.save()
        return self.election
    

class CategoryForm(forms.ModelForm):
    class Meta:
        model = Category
        exclude = ('election', 'slug', 'order')
    class Media:
        js = ('js/jquery.slug.js', )
        css = { 'all': ('css/wizard-forms.css',) }


class CategoryUpdateForm(forms.ModelForm):
    class Meta:
        model = Category
        exclude = ('election', 'slug', 'order')


class QuestionForm(forms.ModelForm):
    new_category = forms.CharField(required=False)

    def __init__(self,*args,**kwargs):
        self.election = kwargs.pop('election')
        super(QuestionForm, self).__init__(*args,**kwargs)
        # Se puede cambiar el queryset del modelo:
        self.fields['category'].queryset = Category.objects.filter(election = self.election)

        choices = [(choice[0], choice[1]) for choice in self.fields['category'].choices]
        empty_choice = choices.pop(0)
        choices.append((empty_choice[0], u'-- Nueva categoría --'))
        self.fields['category'].choices = choices
        self.fields['category'].initial = 0

        # Se puede sobreescribir el required del modelo:
        self.fields['category'].required = False

    def clean(self):
        if 'category' in self.cleaned_data and self.cleaned_data['category'] is None:
            if 'new_category' in self.cleaned_data and len(self.cleaned_data['new_category'].strip()):
                self.cleaned_data['category'], created = Category.objects.get_or_create(name=self.cleaned_data['new_category'], election=self.election)
            else:
                msg = 'Este campo es obligatorio.'
                self._errors['category'] = ErrorList([msg])
                del self.cleaned_data['category']
        return self.cleaned_data


    class Meta:
        model = Question
    class Media:
        css = { 'all': ('css/wizard-forms.css',) }


class AnswerForm(forms.ModelForm):
    class Meta:
        model = Answer
        exclude = ('question')


class PersonalDataForm(forms.ModelForm):
    class Meta:
        model = PersonalData
        exclude = ('election')
    class Media:
        js = ('js/jquery.formset.js', )
        css = { 'all': ('css/wizard-forms.css',) }

class PersonalDataCandidateForm(forms.ModelForm):
    class Meta:
        model = PersonalDataCandidate
        exclude = ('candidate', 'personal_data')
    class Media:
        js = ('js/jquery.formset.js', )

class BackgroundCategoryForm(forms.ModelForm):
    class Meta:
        model = BackgroundCategory
        exclude = ('election')
    class Media:
        js = ('js/jquery.formset.js', )
        css = { 'all': ('css/wizard-forms.css',) }


class BackgroundForm(forms.ModelForm):
    class Meta:
        model = Background
        exclude = ('category')

class BackgroundCandidateForm(forms.ModelForm):
    class Meta:
        model = BackgroundCandidate
        exclude = ('candidate', 'background')
